import asyncio
import aiohttp
import urllib.parse
from threading import Thread

class RulingsObject(object):
	"""
	Master class for all rulings objects.

	Positional arguments:
		No arguments required.

	Optional arguments:
		format : str ... Returns data in the specified method. Defaults to JSON.
		face : str ... If you're using the `image` format, this will specify if
						you want the front or back face.
		version : str ... If you're using the `image` format, this will specify if
						you want the small, normal, large, etc version of the image.
		pretty : str ... Returns a prettier version of the json object. Note that
						  this may break functionality with Scrython.

	Attributes:
		object : str ...... Returns the type of object it is. (card, error, etc)
		had_more : bool ... If true, this ruling object has more rules than it currently displays.
		data : list .................................. A list of ruling objects.
		data_length : int ....................... The length of the `data` list.

		The following require an integer as an arg, which acts as a tuple.
		ruling_object : str ............. The type of object for a given ruling.
		ruling_source : str .......................... The source of the ruling.
		ruling_published_at : str ...... The date when the ruling was published.
		ruling_comment : str ............................. The effective ruling.
	"""
	def __init__(self, _url, **kwargs):
		self.params = {
			'format': kwargs.get('format', 'json'), 'face': kwargs.get('face', ''),
			'version': kwargs.get('version', ''), 'pretty': kwargs.get('pretty', '')
		}

		self.encodedParams = urllib.parse.urlencode(self.params)
		self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

		async def getRequest(client, url, **kwargs):
			async with client.get(url, **kwargs) as response:
				return await response.json()

		async def main(loop):
			async with aiohttp.ClientSession(loop=loop) as client:
				self.scryfallJson = await getRequest(client, self._url)

		def do_everything():
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			loop.run_until_complete(main(loop))

		t = Thread(target=do_everything)
		t.run()

		if self.scryfallJson['object'] == 'error':
			raise Exception(self.scryfallJson['details'])

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This object has no key \'{}\''.format(key))

	def _checkForTupleKey(self, parent, num, key):
		try:
			return self.scryfallJson[parent][num][key]
		except Exception:
			raise KeyError('This ruling has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def has_more(self):
		self._checkForKey('has_more')

		return self.scryfallJson['has_more']

	def data(self):
		self._checkForKey('data')

		return self.scryfallJson['data']

	def data_length(self):
		self._checkForKey('data')

		return len(self.scryfallJson['data'])

	def ruling_object(self, num):
		self._checkForTupleKey('data', num, 'object')

		return self.scryfallJson['data'][num]['object']

	def ruling_source(self, num):
		self._checkForTupleKey('data', num, 'source')

		return self.scryfallJson['data'][num]['source']

	def ruling_published_at(self, num):
		self._checkForTupleKey('data', num, 'published_at')

		return self.scryfallJson['data'][num]['published_at']

	def ruling_comment(self, num):
		self._checkForTupleKey('data', num, 'comment')

		return self.scryfallJson['data'][num]['comment']



from .sets_object import SetsObject

class Sets(SetsObject):
    """
    /sets
    `Sets()` gets it's own special attributes that don't match with the normal set attributes.

    Positional arguments:
        code : str ............................... The 3 letter code of the set.

    Optional arguments:
        All arguments are inherited from SetsObject

    Attributes:
        object : str ... Returns the type of object it is. (card, error, etc)
        has_more : bool ... True if there are more pages available.
        data : list ... List of all data returned.
        data_length : int ... The length of the data returned.

        The following require an integer as an arg, which acts as a tuple.
        set_object(num) : str .................................. The set object.
        set_code(num) : str .............. The three letter set code of the set.
        set_mtgo_code(num) : str .............. The mtgo equivalent of `code()`.
        set_name(num) : str .......................... The full name of the set.
        set_set_type(num) : str ... The type of the set (expansion, commander, etc)
        set_released_at(num) : str .............. The date the set was launched.
        set_block_code(num) : str ... The the letter code for the block the set was in.
        set_block(num) : str .......... The full name of the block a set was in.
        set_parent_set_code(num) : str ........ The set code for the parent set.
        set_card_count(num) : int .............. The number of cards in the set.
        set_digital(num) : bool ..... True if this set is only featured on MTGO.
        set_foil(num) : bool .................. True if this set only has foils.
        set_icon_svg_uri(num) : str ........ A URI to the SVG of the set symbol.
        set_search_uri(num) : str ......... The scryfall API url for the search.

    Example usage:
        >>> set = scrython.sets.Sets()
        >>> set.name(5)
    """
    def __init__(self):
        self._url = 'sets?'
        super(Sets, self).__init__(self._url)

    def _checkForKey(self, key):
        try:
            return self.scryfallJson[key]
        except Exception:
            raise KeyError('This object has no key \'{}\''.format(key))

    def _checkForTupleKey(self, parent, num, key):
        try:
            return self.scryfallJson[parent][num][key]
        except Exception:
            raise KeyError('This object has no key \'{}\''.format(key))

    def object(self):
        self._checkForKey('object')

        return self.scryfallJson['object']

    def has_more(self):
        self._checkForKey('has_more')

        return self.scryfallJson['has_more']

    def data(self):
        self._checkForKey('data')

        return self.scryfallJson['data']

    def data_length(self):
        self._checkForKey('data')

        return len(self.scryfallJson['data'])

    def set_object(self, num):
        self._checkForTupleKey('data', num, 'object')

        return self.scryfallJson['data'][num]['object']

    def set_code(self, num):
        self._checkForTupleKey('data', num, 'code')

        return self.scryfallJson['data'][num]['code']

    def set_mtgo_code(self, num):
        self._checkForTupleKey('data', num, 'mtgo_code')

        return self.scryfallJson['data'][num]['mtgo_code']

    def set_name(self, num):
        self._checkForTupleKey('data', num, 'name')

        return self.scryfallJson['data'][num]['name']

    def set_set_type(self, num):
        self._checkForTupleKey('data', num, 'set_type')

        return self.scryfallJson['data'][num]['set_type']

    def set_released_at(self, num):
        self._checkForTupleKey('data', num, 'released_at')

        return self.scryfallJson['data'][num]['released_at']

    def set_block_code(self, num):
        self._checkForTupleKey('data', num, 'block_code')

        return self.scryfallJson['data'][num]['block_code']

    def set_block(self, num):
        self._checkForTupleKey('data', num, 'block')

        return self.scryfallJson['data'][num]['block']

    def set_parent_set_code(self, num):
        self._checkForTupleKey('data', num, 'parent_set_code')

        return self.scryfallJson['data'][num]['parent_set_code']

    def set_card_count(self, num):
        self._checkForTupleKey('data', num, 'card_count')

        return self.scryfallJson['data'][num]['card_count']

    def set_digital(self, num):
        self._checkForTupleKey('data', num, 'digital')

        return self.scryfallJson['data'][num]['digital']

    def set_foil(self, num):
        self._checkForTupleKey('data', num, 'foil')

        return self.scryfallJson['data'][num]['foil']

    def set_icon_svg_uri(self, num):
        self._checkForTupleKey('data', num, 'icon_svg_uri')

        return self.scryfallJson['data'][num]['icon_svg_uri']

    def set_search_uri(self, num):
        self._checkForTupleKey('data', num, 'search_uri')

        return self.scryfallJson['data'][num]['search_uri']

	#The following attributes are only to override the inherited class attributes.
	#This class has no matching attributes but we still need the getRequest from SetsObject

    def code(self):
        raise AttributeError('This object has no key \'code\'')

    def mtgo_code(self):
        raise AttributeError('This object has no key \'mtgo_code\'')

    def name(self):
        raise AttributeError('This object has no key \'name\'')

    def set_type(self):
        raise AttributeError('This object has no key \'set_type\'')

    def released_at(self):
        raise AttributeError('This object has no key \'released_at\'')

    def block_code(self):
        raise AttributeError('This object has no key \'block_code\'')

    def block(self):
        raise AttributeError('This object has no key \'block\'')

    def parent_set_code(self):
        raise AttributeError('This object has no key \'parent_set_code\'')

    def card_count(self):
        raise AttributeError('This object has no key \'card_count\'')

    def digital(self):
        raise AttributeError('This object has no key \'digital\'')

    def foil(self):
        raise AttributeError('This object has no key \'foil\'')

    def icon_svg_uri(self):
        raise AttributeError('This object has no key \'icon_svg_uri\'')

    def search_uri(self):
        raise AttributeError('This object has no key \'search_uri\'')



from .autocomplete import Autocomplete
from .collector import Collector
from .cardid import Id
from .mtgo import Mtgo
from .multiverse import Multiverse
from .named import Named
from .randomcard import Random
from .search import Search



import asyncio
import aiohttp
import urllib.parse
from threading import Thread

class SetsObject(object):
	"""
	The master class for all sets objects.

	Positional arguments:
		No arguments required.

	Optional arguments:
		format : str ................... The format to return. Defaults to JSON.
		pretty : bool ... Makes the returned JSON prettier. The library may not work properly with this setting.

	Attributes:
		object : str ...... Returns the type of object it is. (card, error, etc)
		code : str ........................ The three letter set code of the set
		mtgo_code : str ........................ The mtgo equivalent of `code()`
		name : str ................................... The full name of the set.
		set_type : str ......... The type of the set (expansion, commander, etc)
		released_at : str ....................... The date the set was launched.
		block_code : str ..... The the letter code for the block the set was in.
		block : str ................... The full name of the block a set was in.
		parent_set_code : str ................. The set code for the parent set.
		card_count : int ......................  The number of cards in the set.
		digital : bool .............. True if this set is only featured on MTGO.
		foil : bool ........................... True if this set only has foils.
		icon_svg_uri : str ................  A URI to the SVG of the set symbol.
		search_uri : str .................. The scryfall API url for the search.
	"""
	def __init__(self, _url, **kwargs):
		self.params = {'format': kwargs.get('format', 'json'), 'pretty': kwargs.get('pretty', '')}

		self.encodedParams = urllib.parse.urlencode(self.params)
		self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

		async def getRequest(client, url, **kwargs):
			async with client.get(url, **kwargs) as response:
				return await response.json()

		async def main(loop):
			async with aiohttp.ClientSession(loop=loop) as client:
				self.scryfallJson = await getRequest(client, self._url)

		def do_everything():
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			loop.run_until_complete(main(loop))

		t = Thread(target=do_everything)
		t.run()

		if self.scryfallJson['object'] == 'error':
			raise Exception(self.scryfallJson['details'])

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This object has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def code(self):
		self._checkForKey('object')

		return self.scryfallJson['code']

	def mtgo_code(self):
		self._checkForKey('mtgo_code')

		return self.scryfallJson['mtgo_code']

	def name(self):
		self._checkForKey('name')

		return self.scryfallJson['name']

	def set_type(self):
		self._checkForKey('set_type')

		return self.scryfallJson['set_type']

	def released_at(self):
		self._checkForKey('released_at')

		return self.scryfallJson['released_at']

	def block_code(self):
		self._checkForKey('block_code')

		return self.scryfallJson['block_code']

	def block(self):
		self._checkForKey('block')

		return self.scryfallJson['block']

	def parent_set_code(self):
		self._checkForKey('parent_set_code')

		return self.scryfallJson['parent_set_code']

	def card_count(self):
		self._checkForKey('card_count')

		return self.scryfallJson['card_count']

	def digital(self):
		self._checkForKey('digital')

		return self.scryfallJson['digital']

	def foil(self):
		self._checkForKey('foil')

		return self.scryfallJson['foil']

	def icon_svg_uri(self):
		self._checkForKey('icon_svg_uri')

		return self.scryfallJson['icon_svg_uri']

	def search_uri(self):
		self._checkForKey('search_uri')

		return self.scryfallJson['search_uri']



from .cards_object import CardsObject

class Collector(CardsObject):
	"""
	cards/collector
	Get a card by collector number.

	Positional arguments:
		code : str ....................... This is the 3 letter code for the set
		collector_number : str ........ This is the collector number of the card

	Optional arguments:
		Inherits all arguments from CardsObject

	Attributes:
		Inherits all attributes from CardsObject

	Example usage:
		>>> card = scrython.cards.Collector(code='exo', collector_number='96')
		>>> card.id()
	"""
	def __init__(self, **kwargs):
		if kwargs.get('code') is None:
			raise TypeError('No code provided to search by')

		self.url = 'cards/{}/{}?'.format(kwargs.get('code'), str(kwargs.get('collector_number')))
		super(Collector, self).__init__(self.url)



#Import classes from cards
from scrython.cards import Autocomplete
from scrython.cards import Collector
from scrython.cards import Id
from scrython.cards import Mtgo
from scrython.cards import Multiverse
from scrython.cards import Named
from scrython.cards import Random
from scrython.cards import Search

#Import classes from rulings
from scrython.rulings import Mtgo
from scrython.rulings import Multiverse
from scrython.rulings import Id
from scrython.rulings import Code

#Import classes from sets
from scrython.sets import Code
from scrython.sets import Sets

#Import classes from catalogs
from scrython.catalog import ArtifactTypes
from scrython.catalog import CardNames
from scrython.catalog import CreatureTypes
from scrython.catalog import EnchantmentTypes
from scrython.catalog import LandTypes
from scrython.catalog import Loyalties
from scrython.catalog import PlaneswalkerTypes
from scrython.catalog import Powers
from scrython.catalog import SpellTypes
from scrython.catalog import Toughnesses
from scrython.catalog import Watermarks
from scrython.catalog import WordBank

#Import symbology
from scrython.symbology import ParseMana
from scrython.symbology import Symbology



from .rulings_object import RulingsObject

class Multiverse(RulingsObject):
    """
    cards/multiverse/:id/rulings

    Gets the ruling of a card by the Multiverse Id.

    Positional arguments:
        id : str ........... The multiverse id of the card you want rulings for.

    Optional arguments:
        All arguments inherited from RulingsObject

    Attributes:
        All attributes inherited from RulingsObject

    Example usage:
        >>> rule = scrython.rulings.Id(id='4301')
        >>> rule.data_length()
    """
    def __init__(self, **kwargs):
        if kwargs.get('id') is None:
            raise TypeError('No id provided to search by')

        self.url = 'cards/multiverse/{}/rulings?'.format(str(kwargs.get('id')))
        super(Multiverse, self).__init__(self.url)



from .artifact_types import ArtifactTypes
from .card_names import CardNames
from .creature_types import CreatureTypes
from .enchantment_types import EnchantmentTypes
from .land_types import LandTypes
from .loyalties import Loyalties
from .planeswalker_types import PlaneswalkerTypes
from .powers import Powers
from .spell_types import SpellTypes
from .toughnesses import Toughnesses
from .watermarks import Watermarks
from .word_bank import WordBank



import asyncio
import aiohttp
import urllib.parse
from threading import Thread

class CatalogsObject(object):
	"""
	Master object for all catalog objects.

	Positional Arguments:
		No arguments are required.

	Optional Arguments:
		format : str ................... The format to return. Defaults to JSON.
		pretty : bool ... Makes the returned JSON prettier. The library may not work properly with this setting.

	Attributes:
		object : str ...... Returns the type of object it is. (card, error, etc)
		uri : str .................. The API URI for the endpoint you've called.
		total_values : int ..................... The number of items in `data()`
		data : list .............. A list of all types returned by the endpoint.
	"""
	def __init__(self, _url, **kwargs):
		self.params = {'format': kwargs.get('format', 'json'), 'pretty': kwargs.get('pretty', '')}

		self.encodedParams = urllib.parse.urlencode(self.params)
		self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

		async def getRequest(client, url, **kwargs):
			async with client.get(url, **kwargs) as response:
				return await response.json()

		async def main(loop):
			async with aiohttp.ClientSession(loop=loop) as client:
				self.scryfallJson = await getRequest(client, self._url)

		def do_everything():
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			loop.run_until_complete(main(loop))

		t = Thread(target=do_everything)
		t.run()

		if self.scryfallJson['object'] == 'error':
			raise Exception(self.scryfallJson['details'])

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This card has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def uri(self):
		self._checkForKey('uri')

		return self.scryfallJson['uri']

	def total_values(self):
		self._checkForKey('total_values')

		return self.scryfallJson['total_values']

	def data(self):
		self._checkForKey('data')

		return self.scryfallJson['data']



import aiohttp
import asyncio
import urllib.parse
from threading import Thread

class CardsObject(object):
	"""
	Master class that all card objects inherit from.

	Positional arguments:
		No positional arguments are required.

	Optional arguments:
		format : str ... Returns data in the specified method. Defaults to JSON.
		face : str ... If you're using the `image` format, this will specify if
						you want the front or back face.
		version : str ... If you're using the `image` format, this will specify if
						you want the small, normal, large, etc version of the image.
		pretty : str ... Returns a prettier version of the json object. Note that
						  this may break functionality with Scrython.
	Attributes:
		object : str  ..... Returns the type of object it is. (card, error, etc)
		id : str  ................................. The scryfall id of the card.
		multiverse_ids : list  ...... The associated multiverse ids of the card.
		mtgo_id : int  ........................ The Magic Online id of the card.
		mtgo_foil_id : int  .............. The Magic Online foil id of the card.
		name : str  ................................. The full name of the card.
		uri : str  .......................... The Scryfall API uri for the card.
		scryfall_uri : str  ................ The full Scryfall page of the card.
		layout : str  ... The image layout of the card. (normal, transform, etc)
		highres_image : bool  ... Returns True if the card has a high res image.
		image_uris : dict  .... All image uris of the card in various qualities.
		cmc : float  ........... A float of the converted mana cost of the card.
		type_line : str  ....................... The full type line of the card.
		oracle_text : str  ................. The official oracle text of a card.
		mana_cost : str  .... The full mana cost using shorthanded mana symbols.
		colors : list  ... An array of strings with all colors found in the mana cost.
		color_identity : list  ... An array of strings with all colors found on the card itself.
		legalities : dict  ..... A dictionary of all formats and their legality.
		reserved : bool  ..... Returns True if the card is on the reserved list.
		reprint : bool  .... Returns True if the card has been reprinted before.
		set_code : str  ............. The 3 letter code for the set of the card.
		set_name : str  ................. The full name for the set of the card.
		set_uri : str  .......... The API uri for the full set list of the card.
		set_search_uri : str  .......................... Same output as set_uri.
		scryfall_set_uri : str  .......... The full link to the set on Scryfall.
		rulings_uri : str  ............ The API uri for the rulings of the card.
		prints_search_uri : str  ... A link to where you can begin paginating all re/prints for this card on Scryfall’s API.
		collector_number : str  .............. The collector number of the card.
		digital : bool  ....... Returns True if the card is the digital version.
		rarity : str  .................................. The rarity of the card.
		illuStringation_id : str  .............. The related id of the card art.
		artist : str  .................................. The artist of the card.
		frame : str  ............................... The year of the card frame.
		full_art : bool  ...... Returns True if the card is considered full art.
		border_color : str  ...................... The color of the card border.
		timeshifted : bool  ........... Returns True if the card is timeshifted.
		colorshifted : bool  ......... Returns True if the card is colorshifted.
		futureshifted : bool  ....... Returns True if the card is futureshifted.
		edhrec_rank : int  .................. The rank of the card on edhrec.com
		currency("<mode>")` : str  ...  Returns currency from modes `usd`, `eur`, and `tix`.
		related_uris : dict  ... A dictionary of related websites for this card.
		purchase_uris : dict  ...... A dictionary of links to purchase the card.
		life_modifier : str  ... This is the cards life modifier value, assuming it's a Vanguard card.
		hand_modifier : str  ... This cards hand modifier value, assuming it's a Vanguard card.
		color_indicator : list  ... An array of all colors found in this card's color indicator.
		all_parts : list  ... This this card is closely related to other cards, this property will be an array with it.
		card_faces : list  ... If it exists, all parts found on a card's face will be found as an object from this array.
		watermark : str  ......... The associated watermark of the card, if any.
		story_spotlight_number : int  ... This card's story spotlight number, if any.
		story_spotlight_uri : str  ... The URI for the card's story article, if any.
		power : str . The power of the creature, if applicable.
		toughness : str . The toughness of the creature, if applicable.
		flavor_text : str . The flavor text of the card, if any.
	"""
	def __init__(self, _url, **kwargs):

		self.params = {
			'format': kwargs.get('format', 'json'), 'face': kwargs.get('face', ''),
			'version': kwargs.get('version', ''), 'pretty': kwargs.get('pretty', '')
		}

		self.encodedParams = urllib.parse.urlencode(self.params)
		self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

		async def getRequest(client, url, **kwargs):
			async with client.get(url, **kwargs) as response:
				return await response.json()

		async def main(loop):
			async with aiohttp.ClientSession(loop=loop) as client:
				self.scryfallJson = await getRequest(client, self._url)

		def do_everything():
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			loop.run_until_complete(main(loop))

		t = Thread(target=do_everything)
		t.run()

		if self.scryfallJson['object'] == 'error':
			raise Exception(self.scryfallJson['details'])

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This card has no key \'{}\''.format(key))

	def _checkForTupleKey(self, parent, num, key):
		try:
			return self.scryfallJson[parent][num][key]
		except Exception:
			raise KeyError('This tuple has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def id(self):
		self._checkForKey('id')

		return self.scryfallJson['id']

	def multiverse_ids(self):
		self._checkForKey('multiverse_ids')

		return self.scryfallJson['multiverse_ids']

	def mtgo_id(self):
		self._checkForKey('mtgo_id')

		return self.scryfallJson['mtgo_id']

	def mtgo_foil_id(self):
		self._checkForKey('mtgo_foil_id')

		return self.scryfallJson['mtgo_foil_id']

	def name(self):
		self._checkForKey('name')

		return self.scryfallJson['name']

	def uri(self):
		self._checkForKey('uri')

		return self.scryfallJson['uri']

	def scryfall_uri(self):
		self._checkForKey('scryfall_uri')

		return self.scryfallJson['scryfall_uri']

	def layout(self):
		self._checkForKey('layout')

		return self.scryfallJson['layout']

	def highres_image(self):
		self._checkForKey('highres_image')

		return self.scryfallJson['highres_image']

	def image_uris(self):
		self._checkForKey('image_uris')

		return self.scryfallJson['image_uris']

	def cmc(self):
		self._checkForKey('cmc')

		return self.scryfallJson['cmc']

	def type_line(self):
		self._checkForKey('type_line')

		return self.scryfallJson['type_line']

	def oracle_text(self):
		self._checkForKey('oracle_text')

		return self.scryfallJson['oracle_text']

	def mana_cost(self):
		self._checkForKey('mana_cost')

		return self.scryfallJson['mana_cost']

	def colors(self):
		self._checkForKey('colors')

		return self.scryfallJson['colors']

	def color_identity(self):
		self._checkForKey('color_identity')

		return self.scryfallJson['color_identity']

	def legalities(self):
		self._checkForKey('legalities')

		return self.scryfallJson['legalities']

	def reserved(self):
		self._checkForKey('reserved')

		return self.scryfallJson['reserved']

	def reprint(self):
		self._checkForKey('reprint')

		return self.scryfallJson['reprint']

	def set_code(self):
		self._checkForKey('set')

		return self.scryfallJson['set']

	def set_name(self):
		self._checkForKey('set_name')

		return self.scryfallJson['set_name']

	def set_uri(self):
		self._checkForKey('set_uri')

		return self.scryfallJson['set_uri']

	def set_search_uri(self):
		self._checkForKey('set_search_uri')

		return self.scryfallJson['set_search_uri']

	def scryfall_set_uri(self):
		self._checkForKey('scryfall_set_uri')

		return self.scryfallJson['scryfall_set_uri']

	def rulings_uri(self):
		self._checkForKey('rulings_uri')

		return self.scryfallJson['rulings_uri']

	def prints_search_uri(self):
		self._checkForKey('prints_search_uri')

		return self.scryfallJson['prints_search_uri']

	def collector_number(self):
		self._checkForKey('collector_number')

		return self.scryfallJson['collector_number']

	def digital(self):
		self._checkForKey('digital')

		return self.scryfallJson['digital']

	def rarity(self):
		self._checkForKey('rarity')

		return self.scryfallJson['rarity']

	def illustration_id(self):
		self._checkForKey('illustration_id')

		return self.scryfallJson['illustration_id']

	def artist(self):
		self._checkForKey('artist')

		return self.scryfallJson['artist']

	def frame(self):
		self._checkForKey('frame')

		return self.scryfallJson['frame']

	def full_art(self):
		self._checkForKey('')

		return self.scryfallJson['full_art']

	def border_color(self):
		self._checkForKey('border_color')

		return self.scryfallJson['border_color']

	def timeshifted(self):
		self._checkForKey('timeshifted')

		return self.scryfallJson['timeshifted']

	def colorshifted(self):
		self._checkForKey('colorshifted')

		return self.scryfallJson['colorshifted']

	def futureshifted(self):
		self._checkForKey('futureshifted')

		return self.scryfallJson['futureshifted']

	def edhrec_rank(self):
		self._checkForKey('edhrec_rank')

		return self.scryfallJson['edhrec_rank']

	def currency(self, mode):
		modes = ['usd', 'eur', 'tix']
		if mode not in modes:
			raise KeyError("{} is not a key.".format(mode))

		self._checkForKey(mode)

		return self.scryfallJson[mode]

	def related_uris(self):
		self._checkForKey('related_uris')

		return self.scryfallJson['related_uris']

	def purchase_uris(self):
		self._checkForKey('purchase_uris')

		return self.scryfallJson['purchase_uris']

	def life_modifier(self):
		self._checkForKey('life_modifier')

		return self.scryfallJson['life_modifier']

	def hand_modifier(self):
		self._checkForKey('hand_modifier')

		return self.scryfallJson['hand_modifier']

	def color_indicator(self):
		self._checkForKey('color_indicator')

		return self.scryfallJson['color_indicator']

	def all_parts(self):
		self._checkForKey('all_parts')

		return self.scryfallJson['all_parts']

	def card_faces(self):
		self._checkForKey('card_faces')

		return self.scryfallJson['card_faces']

	def watermark(self):
		self._checkForKey('watermark')

		return self.scryfallJson['watermark']

	def story_spotlight_number(self):
		self._checkForKey('story_spotlight_number')

		return self.scryfallJson['story_spotlight_number']

	def story_spotlight_uri(self):
		self._checkForKey('story_spotlight_uri')

		return self.scryfallJson['story_spotlight_uri']

	def power(self):
		self._checkForKey('power')

		return self.scryfallJson['power']

	def toughness(self):
		self._checkForKey('toughness')

		return self.scryfallJson['toughness']

	def loyalty(self):
		self._checkForKey('loyalty')

		return self.scryfallJson['loyalty']

	def flavor_text(self):
		self._checkForKey('flavor_text')

		return self.scryfallJson['flavor_text']



from .sets_object import SetsObject

class Code(SetsObject):
    """
    sets/:code
    Get a set with a 3 letter code.

    Positional arguments:
        code : str ............................... The 3 letter code of the set.

    Optional arguments:
        All arguments are inherited from SetsObject

    Attributes:
        All attributes are inherited from SetsObject

    Example usage:
        >>> set = scrython.sets.Code(code='por')
        >>> set.name()
    """
    def __init__(self, code):
        self._url = 'sets/{}?'.format(code)
        super(Code, self).__init__(self._url)



from .cards_object import CardsObject
import urllib.parse

class Named(CardsObject):
	"""
	cards/named
	Gets a card by the name.

	Positional arguments:
		fuzzy : str ................ Uses the fuzzy parameter for the card name.
		or
		exact : str ................ Uses the exact parameter for the card name.

	Optional arguments:
		set : str . Returns the set of the card if specified. Requires the 3 letter set code.
		All arguments are inherited from CardsObject

	Attributes:
		All attributes are inherited from CardsObject

	Example usage:
		>>> card = scrython.cards.Named(exact="Black Lotus")
		>>> card.colors()
	"""
	def __init__(self, **kwargs):
		self.dict = {
			'set':kwargs.get('set', '')
		}

		if kwargs.get('exact') is not None:
			self.dict['exact'] = kwargs.get('exact')

		if kwargs.get('fuzzy') is not None:
			self.dict['fuzzy'] = kwargs.get('fuzzy')

		self.args = urllib.parse.urlencode(self.dict)
		self.url = 'cards/named?' + self.args
		super(Named, self).__init__(self.url)



import asyncio
import aiohttp
import urllib.parse
from threading import Thread

class SymbologyObject(object):
	"""
	The master class for all symbology objects.

	Positional arguments:
		No arguments required.

	Optional arguments:
		format : str ................... The format to return. Defaults to JSON.
		pretty : bool ... Makes the returned JSON prettier. The library may not work properly with this setting.

	Attributes:
		No attributes to call.
	"""
	def __init__(self, _url, **kwargs):
		self.params = {'format': kwargs.get('format', 'json'), 'pretty': kwargs.get('pretty', '')}

		self.encodedParams = urllib.parse.urlencode(self.params)
		self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

		async def getRequest(client, url, **kwargs):
			async with client.get(url, **kwargs) as response:
				return await response.json()

		async def main(loop):
			async with aiohttp.ClientSession(loop=loop) as client:
				self.scryfallJson = await getRequest(client, self._url)

		def do_everything():
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			loop.run_until_complete(main(loop))

		t = Thread(target=do_everything)
		t.run()

		if self.scryfallJson['object'] == 'error':
			raise Exception(self.scryfallJson['details'])



from .cards_object import CardsObject
import urllib.parse

class Autocomplete(CardsObject):
	"""
	cards/autocomplete
	Get a list of potential autocompletion phrases.

	Positional arguments:
		q : str ........... The query of the autocompletion.

	Optional arguments:
		Inherits arguments from CardsObject.

	Attributes:
		object : str ........ Returns what kind of object it is.
		total_items : int ...... How many items are in the list.
		data : list ....... The list of potential autocompletes.

	Example usage:
		>>> auto = scrython.cards.Autocomplete(q="Thal")
		>>> auto.total_items()
	"""
	def __init__(self, **kwargs):
		if kwargs.get('q') is None:
			raise TypeError('No query provided to search by')

		self.dict = {
			'q': kwargs.get('q'),
			'pretty': kwargs.get('pretty', 'false'),
			'format': kwargs.get('format', 'json')
			}

		self.args = urllib.parse.urlencode(self.dict)
		self.url = 'cards/autocomplete?' + self.args
		super(Autocomplete, self).__init__(self.url)

	def object(self):
		super(Autocomplete, self)._checkForKey('object')

		return self.scryfallJson['object']

	def total_items(self):
		super(Autocomplete, self)._checkForKey('total_items')

		return self.scryfallJson['total_items']

	def data(self):
		super(Autocomplete, self)._checkForKey('data')

		return self.scryfallJson['data']

	#The following attributes are only to override the inherited class attributes.
	#This class has no matching attributes but we still need the getRequest function from CardsObject

	def id(self):
		raise AttributeError('Search object has no attribute \'id\'')

	def multiverse_ids(self):
		raise AttributeError('Search object has no attribute \'multiverse_ids\'')

	def mtgo_id(self):
		raise AttributeError('Search object has no attribute \'mtgo_id\'')

	def mtgo_foil_id(self):
		raise AttributeError('Search object has no attribute \'mtgo_foil_id\'')

	def name(self):
		raise AttributeError('Search object has no attribute \'name\'')

	def uri(self):
		raise AttributeError('Search object has no attribute \'uri\'')

	def scryfall_uri(self):
		raise AttributeError('Search object has no attribute \'scryfall_uri\'')

	def layout(self):
		raise AttributeError('Search object has no attribute \'layout\'')

	def highres_image(self):
		raise AttributeError('Search object has no attribute \'highres_image\'')

	def image_uris(self):
		raise AttributeError('Search object has no attribute \'image_uris\'')

	def cmc(self):
		raise AttributeError('Search object has no attribute \'cmc\'')

	def type_line(self):
		raise AttributeError('Search object has no attribute \'type_line\'')

	def oracle_text(self):
		raise AttributeError('Search object has no attribute \'oracle_text\'')

	def mana_cost(self):
		raise AttributeError('Search object has no attribute \'mana_cost\'')

	def colors(self):
		raise AttributeError('Search object has no attribute \'colors\'')

	def color_identity(self):
		raise AttributeError('Search object has no attribute \'color_identity\'')

	def legalities(self):
		raise AttributeError('Search object has no attribute \'legalities\'')

	def reserved(self):
		raise AttributeError('Search object has no attribute \'reserved\'')

	def reprint(self):
		raise AttributeError('Search object has no attribute \'reprint\'')

	def set_code(self):
		raise AttributeError('Search object has no attribute \'	def set_code\'')

	def set_name(self):
		raise AttributeError('Search object has no attribute \'	def set_name\'')

	def set_uri(self):
		raise AttributeError('Search object has no attribute \'set_uri\'')

	def set_search_uri(self):
		raise AttributeError('Search object has no attribute \'set_search_uri\'')

	def scryfall_set_uri(self):
		raise AttributeError('Search object has no attribute \'scryfall_set_uri\'')

	def rulings_uri(self):
		raise AttributeError('Search object has no attribute \'rulings_uri\'')

	def prints_search_uri(self):
		raise AttributeError('Search object has no attribute \'prints_search_uri\'')

	def collector_number(self):
		raise AttributeError('Search object has no attribute \'collector_number\'')

	def digital(self):
		raise AttributeError('Search object has no attribute \'digital\'')

	def rarity(self):
		raise AttributeError('Search object has no attribute \'rarity\'')

	def illustration_id(self):
		raise AttributeError('Search object has no attribute \'illustration_id\'')

	def artist(self):
		raise AttributeError('Search object has no attribute \'artist\'')

	def frame(self):
		raise AttributeError('Search object has no attribute \'frame\'')

	def full_art(self):
		raise AttributeError('Search object has no attribute \'full_art\'')

	def border_color(self):
		raise AttributeError('Search object has no attribute \'border_color\'')

	def timeshifted(self):
		raise AttributeError('Search object has no attribute \'timeshifted\'')

	def colorshifted(self):
		raise AttributeError('Search object has no attribute \'colorshifted\'')

	def futureshifted(self):
		raise AttributeError('Search object has no attribute \'futureshifted\'')

	def edhrec_rank(self):
		raise AttributeError('Search object has no attribute \'edhrec_rank\'')

	def currency(self, mode):
		raise AttributeError('Search object has no attribute \'currency(self,\'')

	def related_uris(self):
		raise AttributeError('Search object has no attribute \'related_uris\'')

	def purchase_uris(self):
		raise AttributeError('Search object has no attribute \'purchase_uris\'')

	def life_modifier(self):
		raise AttributeError('Search object has no attribute \'life_modifier\'')

	def hand_modifier(self):
		raise AttributeError('Search object has no attribute \'hand_modifier\'')

	def color_indicator(self):
		raise AttributeError('Search object has no attribute \'color_indicator\'')

	def all_parts(self):
		raise AttributeError('Search object has no attribute \'all_parts\'')

	def card_faces(self):
		raise AttributeError('Search object has no attribute \'card_faces\'')

	def watermark(self):
		raise AttributeError('Search object has no attribute \'watermark\'')

	def story_spotlight_number(self):
		raise AttributeError('Search object has no attribute \'story_spotlight_number\'')

	def story_spotlight_uri(self):
		raise AttributeError('Search object has no attribute \'story_spotlight_uri\'')



from .symbology_object import SymbologyObject

class Symbology(SymbologyObject):
	"""
	/symbology

	Positional arguments:
		No arguments are required.

	Optional arguments:
		All arguments are inherited from SymbologyObject

	Attributes:
		object : str . Returns the type of object it is. (card, error, etc)
		has_more : bool . True if there are more pages to the object.
		data : list . A list of all data returned.
		data_length : int . The length of the data returned.

		The following require an integer as an arg, which acts as a tuple.
		symbol_symbol(num) : str . The plaintext symbol, usually written with curly braces.
		symbol_loose_variant(num) : str . The alternate version of the symbol, without curly braces.
		symbol_transposable(num): bool . True if it's possibly to write the symbol backwards.
		symbol_represents_mana(num): bool . True if this is a mana symbol.
		symbol_cmc(num): float . The total converted mana cost of the symbol.
		symbol_appears_in_mana_costs(num): bool . True if the symbol appears on the mana cost of any card.
		symbol_funny(num): bool . True if the symbol is featured on any funny cards.
		symbol_colors(num): float . An array of all colors in the given symbol.

	Example usage:
		>>> symbol = scrython.symbology.Symbology()
	"""
	def __init__(self):
		self.url = 'symbology?'
		super(Symbology, self).__init__(self.url)

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This object ahs no key \'{}\''.format(key))

	def _checkForTupleKey(self, parent, num, key):
		try:
			return self.scryfallJson[parent][num][key]
		except Exception:
			raise KeyError('This object has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def has_more(self):
		self._checkForKey('has_more')

		return self.scryfallJson['has_more']

	def data(self):
		self._checkForKey('has_more')

		return self.scryfallJson['data']

	def data_length(self):
		self._checkForKey('data')

		return len(self.scryfallJson['data'])

	def symbol_symbol(self, num):
		self._checkForTupleKey('data', num, 'symbol')

		return self.scryfallJson['data'][num]['symbol']

	def symbol_loose_variant(self, num):
		self._checkForTupleKey('data', num, 'loose_variant')

		return self.scryfallJson['data'][num]['loose_variant']

	def symbol_transposable(self, num):
		self._checkForTupleKey('data', num, 'transposable')

		return self.scryfallJson['data'][num]['transposable']

	def symbol_represents_mana(self, num):
		self._checkForTupleKey('data', num, 'represents_mana')

		return self.scryfallJson['data'][num]['represents_mana']

	def symbol_cmc(self, num):
		self._checkForTupleKey('data', num, 'cmc')

		return self.scryfallJson['data'][num]['cmc']

	def symbol_appears_in_mana_costs(self, num):
		self._checkForTupleKey('data', num, 'appears_in_mana_costs')

		return self.scryfallJson['data'][num]['appears_in_mana_costs']

	def symbol_funny(self, num):
		self._checkForTupleKey('data', num, 'funny')

		return self.scryfallJson['data'][num]['funny']

	def symbol_colors(self, num):
		self._checkForTupleKey('data', num, 'colors')

		return self.scryfallJson['data'][num]['colors']



from .cards_object import CardsObject

class Multiverse(CardsObject):
	"""
	cards/multiverse
	Get a card by Multiverse id

	Positional arguments:
		id : str ....... This is the associated multiverse id of the given card.

	Optional arguments:
		Inherits all arguments from CardsObject

	Attributes:
		Inherits all attributes from CardsObject

	Example usage:
		>>> card = scrython.cards.Multiverse(id='96865')
		>>> card.name()
	"""
	def __init__(self, **kwargs):
		if kwargs.get('id') is None:
			raise TypeError('No id provided to search by')

		self.url = 'cards/multiverse/{}?'.format(str(kwargs.get('id')))
		super(Multiverse, self).__init__(self.url)



from .cards_object import CardsObject

class Id(CardsObject):
	"""
	cards/id
	Get a card by the Scryfall id.

	Positional arguments:
		id : str ....................... The Scryfall Id of the card.

	Optional arguments:
		Inherits all arguments from CardsObject.

	Attributes:
		All attributes are inherited from CardsObject.

	Example usage:
		>>> card = scrython.cards.Id(id="5386a61c-4928-4bd1-babe-5b089ab8b2d9")
		>>> card.name()
	"""
	def __init__(self, **kwargs):
		if kwargs.get('id') is None:
			raise TypeError('No id provided to search by')

		self.url = 'cards/{}?'.format(str(kwargs.get('id')))
		super(Id, self).__init__(self.url)



from setuptools import setup

readme = ''
with open('README.md') as f:
	readme = f.read()

setup(
	name='scrython',
	packages=['scrython', 'scrython.cards', 'scrython.rulings', 'scrython.catalog', 'scrython.sets', 'scrython.symbology'],
	version='1.4.1',
	description='A wrapper for using the Scryfall API.',
	long_description=readme,
	url='https://github.com/NandaScott/Scrython',
	download_url='https://github.com/NandaScott/Scrython/archive/0.1.0.tar.gz',
	author='Nanda Scott',
	author_email='nanda1123@gmail.com',
	license='MIT',
	keywords=['Scryfall', 'magic', 'the gathering', 'scrython', 'wrapper'],
	install_requires=['aiohttp', 'asyncio']
	)



from .cards_object import CardsObject


class Random(CardsObject):
    """
    cards/random
    Get a random card.

    Positional arguments:
        No arguments are required.

    Optional arguments:
        All arguments are inherited from CardsObject

    Attributes:
        All attributes are inherited from CardsObject

    Example usage:
        >>> card = scrython.cards.Random()
        >>> card.purchase_uris()
    """
    def __init__(self):
        self.url = 'cards/random?'
        super(Random, self).__init__(self.url)



from .rulings_object import RulingsObject

class Code(RulingsObject):
    """
    cards/:code/:collector_number/rulings

    Gets the ruling of a card by the set and collector number.

    Positional arguments:
        set : str ...... The 3 letter set code of the card you want rulings for.
        collector_number : ................... The collector number of the card.

    Optional arguments:
        All arguments inherited from RulingsObject

    Attributes:
        All attributes inherited from RulingsObject

    Example usage:
        >>> rule = scrython.rulings.Code(code='ddg', collector_number='42')
        >>> rule.data_length()
    """
    def __init__(self, code, collector_number):
        self.url = 'cards/{}/{}/rulings?'.format(code.lower(), str(collector_number))
        super(Code, self).__init__(self.url)



from .catalogs_object import CatalogsObject

class SpellTypes(CatalogsObject):
    """
    catalogs/spell-types

    Catalog object for all known spell types

    Positional Arguments:
        No arguments are required.

    Optional Arguments:
        All arguments are inherited from CatalogsObject

    Attributes:
        All attributes are inherited from CatalogsObject

    Example usage:
        >>> catalog = scrython.catalog.SpellTypes()
        >>> catalog.data()
    """
    def __init__(self):
        self._url = 'catalog/spell-types?'
        super(SpellTypes, self).__init__(self._url)



from .symbology_object import SymbologyObject

class ParseMana(SymbologyObject):
	"""
	symbology/parse-mana

	Positional arguments:
		cost : str ....................... The given mana cost you want. (`RUG`)

	Optional arguments:
		All arguments are inherited from SymbologyObject

	Attributes:
		object : str ...... Returns the type of object it is. (card, error, etc)
		mana_cost : str ............................... The formatted mana cost.
		cmc : float ....................... The converted mana cost of the card.
		colors : list ................... A list of all colors in the mana cost.
		colorless : bool ................... True if the mana cost is colorless.
		monocolored : bool .............. True if the mana cost is mono colored.
		multicolored : bool ...... True if the mana cost is a multicolored cost.

	Example usage:
		>>> mana = scrython.symbology.ParseMana(cost="xcug")
		>>> mana.colors()
	"""
	def __init__(self, cost):
		self.cost = cost
		self.url = 'symbology/parse-mana?cost=' + self.cost
		super(ParseMana, self).__init__(self.url)

	def _checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except Exception:
			raise KeyError('This object has no key \'{}\''.format(key))

	def object(self):
		self._checkForKey('object')

		return self.scryfallJson['object']

	def mana_cost(self):
		self._checkForKey('cost')

		return self.scryfallJson['cost']

	def cmc(self):
		self._checkForKey('cmc')

		return self.scryfallJson['cmc']

	def colors(self):
		self._checkForKey('colors')

		return self.scryfallJson['colors']

	def colorless(self):
		self._checkForKey('colorless')

		return self.scryfallJson['colorless']

	def monocolored(self):
		self._checkForKey('monocolored')

		return self.scryfallJson['monocolored']

	def multicolored(self):
		self._checkForKey('multicolored')

		return self.scryfallJson['multicolored']



from .cards_object import CardsObject
import urllib.parse

class Mtgo(CardsObject):
	"""
	cards/mtgo
	Get a card by MTGO id.

	Positional arguments:
		id : str ............................. The required mtgo id of the card.

	Optional arguments:
		All arguments are inherited from CardsObject

	Attributes:
		All attributes are inherited from CardsObject

	Example usage:
		>>> card = scrython.cards.Mtgo(id="48296")
		>>> card.set_name()
	"""
	def __init__(self, **kwargs):
		if kwargs.get('id') is None:
			raise TypeError('No id provided to search by')

		self.url = 'cards/mtgo/{}?'.format(str(kwargs.get('id')))
		super(Mtgo, self).__init__(self.url)
